
#include <map>
#include <vector>
#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <string>
#include <regex.h>
using namespace std;

//./blastall -p blastn -i ../blast-query/q1.query -o ../blast-output/output.txt -d ../blast-data/aeropyrum-pernix.fa -m 5


#define FIVEPRIMETO3PRIME 5
#define THREEPRIMETO5PRIME 3
#define __nmatch__ 1
#define __query_start_pattern__ "QUERY[ ]+[0-9]+[ ]+[agct-]+[ ]+[0-9]+"
#define __align_pattern__ "[0-9]+[ ]+[agct-]+[ ]+[0-9]+"
struct pregMatch
{
  int start;
  int end;
  string match;
};

typedef pregMatch hitRecord;

pregMatch pregMatchOnce(string _haystack,string _needle)
{
  regex_t preg;
  pregMatch pm;
  pm.start=-1;
  
  regmatch_t pmatch[__nmatch__];
  unsigned int i,len;

  memset(&preg,sizeof(regex_t),0);

  if(regcomp(&preg,_needle.c_str(),REG_EXTENDED)!=0)
    return pm;

  if(regexec(&preg,_haystack.c_str(),__nmatch__,pmatch,0)!=0)
    return pm;

  for(i=0;i<__nmatch__;i++)
  {
    if(pmatch[i].rm_so == -1) continue;

    len=(pmatch[i].rm_eo - pmatch[i].rm_so);
    pm.start=pmatch[i].rm_so;
    pm.end=pmatch[i].rm_eo;
    pm.match=(_haystack.substr(pmatch[i].rm_so,len));
  }

  return pm;
}


struct hitRecordSet
{
  multimap<int,hitRecord> rec;
  typedef multimap<int,hitRecord>::iterator iterator;
};


hitRecordSet blast(string queryName,string blastPath,string blastQueryPath,string
blastOutputPath,string blastDataBase,string queryString,bool _echo=false)
{
     strVector vec; 
     hitRecord rec;
     vector<hitRecord> hitRec;
     hitRecordSet hitRecMap;
  ofstream ofile;
  ofile.open((blastQueryPath+"/"+queryName+".in").c_str());
  ofile<<">"<<queryName<<endl;
  ofile<<queryString;
  ofile.close();
  
  string cmd;
  cmd=blastPath+" -p blastn -i "+blastQueryPath+"/"+
      queryName+".in -o "+blastOutputPath+"/"+queryName+".out -d "+
      blastDataBase+" -m 5" ;


  system(cmd.c_str());

  
  ifstream ifile;
  ifile.open((blastOutputPath+"/"+queryName+".out").c_str());

  char tmpstr[1024];

  pregMatch pm;

  bool relevant=false;
  
  while(true)
  {
    if(ifile.eof())
      break;

    ifile.getline(tmpstr,1024);

    //cout<<">>"<<tmpstr<<endl;

    if(!relevant)
    {

      pm=pregMatchOnce(tmpstr,__query_start_pattern__);
      if(pm.start!=-1)
        relevant=true;
    }else
    {

      pm=pregMatchOnce(tmpstr,__align_pattern__);
    }

      
    /*if(pm.start!=-1)
      cout<<"++"<<pm.match<<endl;*/
      
  	
      int j;
      
      if(pm.start!=-1)
      {
      	vec=str_split(pm.match," ");
	j=vec.size();
	rec.start=atoi(vec[j-3].c_str());
	rec.end=atoi(vec[j-1].c_str());
	rec.match=vec[j-2];
	//cout<<"!!:"<<vec.size()<<"!!::"<<pm.match<<"===>"<<rec.start<<"->"<<rec.end<<":"<<rec.match<<endl;
        hitRec.push_back(rec);
	hitRecMap.rec.insert(map<int,hitRecord>::value_type(rec.start,rec));
      } 
      
  }
  
  /*for(vector<hitRecord>::iterator i=hitRec.begin();
  	i!=hitRec.end();
	i++)
	{
	  
	  cout<<(*i).start<<"->"<<(*i).end<<":"<<(*i).match<<endl;
	  
	
	}*/

if(_echo)
{	
  for(hitRecordSet::iterator j=hitRecMap.rec.begin();
  	j!=hitRecMap.rec.end();
	j++)
	{
		cout<<(*j).second.start<<"->"<<(*j).second.end<<":"<<(*j).second.match<<endl;
	}
	
}
  ifile.close();
  
   /* char buffer[5000];
    ifstream fin;
    fin.open(blastDataBase.c_str());
    while(!fin.eof())
    {
      if(fin.eof())
      	break;
	
    fin.getline(buffer,5000);
    cout<<buffer<<endl;
    }
    fin.close();*/
    
    return hitRecMap;
}

string reverseString(const string& _str)
{
	string reta;
	for(int i=_str.length()-1;i>=0;i--)
		reta+=_str[i];
		
		
	return reta;

}
double scoreSequence (string _str,vector<double>& svec,double mismatchK,double matchK,int orientation=FIVEPRIMETO3PRIME)
{
  double score=0;
  /*for(int i=0;i<_str.length();i++)
  {
  	score+=(_str[i]=='-')?0:pow((double)1.25,(double)((orientation==FIVEPRIMETO3PRIME)?i:_str.length()-i-1));
 	
 
  }*/

/*  int i=0;
  int k;
  	int len=_str.length();
		cout<<"len="<<len<<endl;	
  	for( i=((orientation==FIVEPRIMETO3PRIME)?len-1:0);
		((orientation==FIVEPRIMETO3PRIME)?i>=0:i<len);
		i=((FIVEPRIMETO3PRIME)?i-1:i+1))
 	 {
  		score+=(_str[i]=='-')?0:pow((double)1.25,(double)len-i);
		k=((orientation==FIVEPRIMETO3PRIME)?len-i-1:i);
		cout<<"ka="<<k<<endl;	
		if(k>=svec.size())
		{
			svec.push_back(1);
		}
		else
		{ int i=0;
		
			svec[k]++;
		}
 	 }
  
*/
	
	if(FIVEPRIMETO3PRIME)
		_str=reverseString(_str);
		
	int len=_str.length();
	
	for(int i=0;i<len;i++)
	{
		score+=(_str[i]=='-')?-1*pow((double)mismatchK,(double)len-i):pow((double)matchK,(double)len-i);
		if(i>=svec.size())
		{	
		
			svec.push_back((_str[i]=='-')?0:1);
		}
		else
		{ 
			if(_str[i]!='-')
				svec[i]++;
		}
	}
	
	return score;

}


double scorePrimer( hitRecordSet& hrs,vector<double>& svec,double mismatchK,double matchK,int orientation=FIVEPRIMETO3PRIME)
{
   double score=0;

   
   
   for(hitRecordSet::iterator i=hrs.rec.begin();
   	i!=hrs.rec.end();
	i++)
	{
	  score+=scoreSequence((*i).second.match,svec,mismatchK,matchK/*,orientation*/);
	  
	
	}
   return score;

}


