#include <iostream>
#include <stdlib.h>
#include <string>
#include <fstream>
//#include "primerstack.h"
#include "biostring.h"
#include "primertype.h"
#include "dictionary.h"
#include <stack>

using namespace std;

bool match_L_seq(string line){
	int len=line.size();
	if (line.substr(0,20)=="PRIMER_LEFT_SEQUENCE")
		return true;
	if (line.substr(0,12)=="PRIMER_LEFT_" && line.substr(len-8,8)=="SEQUENCE")
		return true;
	return false;
}

bool match_R_seq(string line){
	int len=line.size();
	if (line.substr(0,21)=="PRIMER_RIGHT_SEQUENCE")
		return true;
	if (line.substr(0,13)=="PRIMER_RIGHT_" && line.substr(len-8,8)=="SEQUENCE")
		return true;
	return false;
}


bool match_L_pos_len(string line){
	int len=line.size();
	if(line=="PRIMER_LEFT")
		return true;
	if(line.substr(0,12)=="PRIMER_LEFT_" && line[len-1]>='0' && line[len-1]<='9')
		return true;
	return false;
}

bool match_R_pos_len(string line){
	int len=line.size();
	if(line=="PRIMER_RIGHT")
		return true;
	if(line.substr(0,13)=="PRIMER_RIGHT_" && line[len-1]>='0' && line[len-1]<='9')
		return true;
	return false;
}

void get_no(string line, int& pos, int& len){ 
	int length=line.size();
	int i;
	int com=0;
	for(i=0; i<length; i++){
		if(line[i]==','){
			com=i;
		    break;
		};
	};

	string p,l;
	p=line.substr(0,i);
	l=line.substr(i+1,length-i+1);
    
	pos=atoi(p.c_str());
	len=atoi(l.c_str());
 
}

struct primerSet
{
  vector<Primer_data> leftPrimers;
  vector<Primer_data> rightPrimers;
  typedef vector<Primer_data>::iterator iterator;
};

void resetPrimerSet(primerSet &ps)
{
	ps.leftPrimers.clear();
	ps.rightPrimers.clear();
}

primerSet primer_main(string _seqfile,int _snploc,int _garanteesize,string _configfile,string _inputfile,string _outputfile,string _primer3corepath,bool _echoresult=false){

	ifstream primer3_out;
	
	/////////////////////////////	
	ifstream seqfile,config;
	ofstream input;
	
	seqfile.open(_seqfile.c_str());
	config.open(_configfile.c_str());
	input.open(_inputfile.c_str());
	
	int target=_snploc;
	int gs=_garanteesize;
    
	string cline,sline;
	char clines[1024];
	while(!config.eof()){
	
		if(config.eof())
			break;
			
		//config>>cline;
		
		//input<<cline<<endl;
		config.getline(clines,1024);
		
		//cerr<<clines<<endl;
		if(strcmp(clines,""))
			input<<clines<<endl;
	};
    
	seqfile>>sline;
	input<<sline<<endl;
	input<<"TARGET="<<(target-gs)<<","<<2*gs+1;
	input<<endl;
	input<<"=";

	config.close();
	input.close();
	seqfile.close();

	system((_primer3corepath+" < "+ _inputfile + " > " + _outputfile).c_str());
	///////////////////////////
	
	primer3_out.open(_outputfile.c_str());

	string line;
	primerSet reta;
	
	stack<NucleoString> primerR,primerL;
	stack<int> posL,posR,lenL,lenR;

	while(!primer3_out.eof()){
		primer3_out>>line;
		dictionary outdata;
		outdata.convert(line);
		
		if (match_L_seq(outdata.return_key())){
			primerL.push(outdata.return_value());
		};

		if (match_R_seq(outdata.return_key())){
            primerR.push(outdata.return_value()); 
		};

		if (match_L_pos_len(outdata.return_key())){
			int l,p;
			get_no(outdata.return_value(),p,l);
			posL.push(p);
			lenL.push(l);
		};

		if (match_R_pos_len(outdata.return_key())){
			int l,p;
			get_no(outdata.return_value(),p,l);
			posR.push(p);
			lenR.push(l);
		};
	};

	while(!primerL.empty()){
		Primer_data Left_primer(posL.top(),lenL.top(),primerL.top());
		posL.pop();
		lenL.pop();
		primerL.pop();
		reta.leftPrimers.push_back(Left_primer);
	};

	while(!primerR.empty()){
		Primer_data Right_primer(posR.top(),lenR.top(),primerR.top());
		posR.pop();
		lenR.pop();
		primerR.pop();
		reta.rightPrimers.push_back(Right_primer);
	};
	
	
	
	if(_echoresult)
	{
	cout<<"Left primer:"<<endl;
	for(primerSet::iterator i=reta.leftPrimers.begin();
	    i!=reta.leftPrimers.end();
	    i++
	    ){
		Primer_data LP((*i));
		cout<<LP.position<<" "<<LP.len<<" ";
		LP.seq.Output();
	      };

	cout<<"Right primer:"<<endl;
	for(primerSet::iterator i=reta.rightPrimers.begin();
	    i!=reta.rightPrimers.end();
	    i++
	    ){
		Primer_data RP((*i));
		cout<<RP.position<<" "<<RP.len<<" ";
		RP.seq.Output();
	      };
	}
	
	primer3_out.close();
	//system("PAUSE");
	return reta;
}


