#ifndef __FASTA_LOADER
#define __FASTA_LOADER

#include "../interpreter/bas.h"
#include <fstream>
#include <string>

#define FASTALOADER_BUFFER_SIZE 1024
#define FASTALOADER_DEFAULT_LINE_LENGTH 70
#define FASTALOADER_HEADER_KEY "header"
#define FASTALOADER_SEQUENCE_KEY "sequence"


using namespace std;

class fastaLoader: public configIO<string,string>
{
  private:
  int globalLocAdj;
  public:
  bool loadFastaFile(const string& _filename,int _from=-1,int _length=-1);
  void saveFastaFile(const string& _filename,int _linelength=FASTALOADER_DEFAULT_LINE_LENGTH);
  string getHeader(); 
  string getSequence(); 
  string getGI(); 
  string getRef(); 
  string getDesc(); 
  
  void setHeader(const string& _header);
  void setSequence(const string& _sequence);
  void setGI(const string& _gi);
  void setRef(const string& _ref);
  void setDesc(const string& _desc);
  
  int local2Global(int _loc);
  int global2Local(int _loc);

};


int fastaLoader::local2Global(int _loc)
{
  return _loc+globalLocAdj;
}

int fastaLoader::global2Local(int _loc)
{
  return _loc-globalLocAdj;
}



string fastaLoader::getHeader() 
{
  return (*this)(FASTALOADER_HEADER_KEY);
}

string fastaLoader::getSequence() 
{
  return (*this)(FASTALOADER_SEQUENCE_KEY);
}

void fastaLoader::setSequence(const string& _sequence)
{
  (*this)(FASTALOADER_SEQUENCE_KEY)=_sequence;
}

void fastaLoader::setHeader(const string& _header)
{
  (*this)(FASTALOADER_HEADER_KEY)=_header;
}

bool fastaLoader::loadFastaFile(const string& _filename,int _from,int _length)
{

  ifstream fin;
  char buffer[FASTALOADER_BUFFER_SIZE];
  fin.open(_filename.c_str());
  fin.getline(buffer,FASTALOADER_BUFFER_SIZE);
  
  int cpos=0;
  int ppos=0;
  int fcpos=0;
  int fppos=0;
  
  int lengthgot=0;
  string toadd="";
  
  int linelength;
  
  if(buffer[0]!='>')
  	return false;
  
  (*this)(FASTALOADER_HEADER_KEY)=buffer;
  setSequence("");
  
    
  if(_from==-1 && _length==-1)
  {
    globalLocAdj=0;
    while(!fin.eof())	
    {
      if(fin.eof())
        break;
     
      fin.getline(buffer,FASTALOADER_BUFFER_SIZE);
      (*this)(FASTALOADER_SEQUENCE_KEY)+=buffer;
    }    
    	
  }else
  {
//    cout<<"start loading (x,y)"<<endl;
   fcpos=fin.tellg();
    _from=(_from>0)?_from:0;
    globalLocAdj=_from;
    cout<<"fasta loading "<<_filename<<" ("<<_from<<","<<_length<<")"<<endl;
      
    while(!fin.eof())	
    {
      if(fin.eof())
        break;
      
      fppos=fcpos;
      ppos=cpos;
 //         cout<<"!!!"<<fcpos<<endl;
      fin.getline(buffer,FASTALOADER_BUFFER_SIZE);
      cpos+=strlen(buffer);
         
      fcpos=fin.tellg();
      
      
      if(cpos>_from && lengthgot==0)
      {
        fcpos=fppos;
//	cout<<">>!!!"<<fcpos<<endl;
	fin.seekg(fcpos);
	toadd=((string)(buffer)).substr(_from-ppos,_length-lengthgot);
	(*this)(FASTALOADER_SEQUENCE_KEY)+=toadd;
	
	lengthgot+=toadd.length();
        continue;
      }
            
      if(lengthgot>0) 
      {

	//cout<<"pos="<<(_from-cpos)<<endl;
        //only activated after cpos>_from action
	toadd=((string)(buffer)).substr(0,_length-lengthgot);
	(*this)(FASTALOADER_SEQUENCE_KEY)+=toadd;	
	lengthgot+=toadd.length();
	//cout<<"lengthgot="<<lengthgot<<endl;
	if(lengthgot==_length)
	  break;
      }
      

    }    
  
  
  }
  
  
  fin.close();
  return true;
}

void fastaLoader::saveFastaFile(const string& _filename,int _linelength)
{
  ofstream fout;
  fout.open(_filename.c_str());
  fout<<(*this)(FASTALOADER_HEADER_KEY);
  
  strVector seqChopped;
  seqChopped=chopString((*this)(FASTALOADER_SEQUENCE_KEY),_linelength);
  for(strVector::iterator i=seqChopped.begin();
      i!=seqChopped.end();
      i++)
      {
        fout<<endl<<(*this)(*i);
      
      }
      
  fout.close();
}





#endif